package foras.display {
	public class ForHSLColor {
		private var luminance:Number;
		private var saturation:Number;
		private var hue:Number;
 
		/**
		 * Contructs an HSL object for containing Hue Saturation and Luminance values, and
		 * converting them to an RGB or Hex color
		 * @param	h Hue (0-1)
		 * @param	s Saturation(0-1)
		 * @param	l Luminance(0-1)
		 */
		public function ForHSLColor(h:Number=0, s:Number=1, l:Number=.5) {
			hue = h;
			saturation = s;
			luminance = l;
		}
 
		/*
		 * Getters for each channel
		 */
		public function get Hue():Number {
			return hue;
		}
		public function get Saturation():Number {
			return saturation;
		}
		public function get Luminance():Number {
			return luminance;
		}
		/*
		 *Setters for each channel
		 */
		public function set Hue(Value:Number):void {
			hue = (Value>1)? 1 : ((Value<0)?0:Value);
		}
		public function set Saturation(Value:Number):void {
			saturation = (Value>1)? 1 : ((Value<0)?0:Value);
		}
		public function set Luminance(Value:Number):void {
			luminance = (Value>1)? 1 : ((Value<0)?0:Value);
		}
		
		public function clone():ForHSLColor {
			return new ForHSLColor(hue, saturation, luminance);
		}
		
		/**
		 * Converts the HSL object to a Hex value defining the same color
		 * @return Hex value of HSL color
		 */
		public function getRGB():Number{
			var r:Number;
			var g:Number;
			var b:Number;
 
			if(luminance==0)
			{
				r=g=b=0;
			}
			else
			{
				if(saturation == 0)
				{
					r=g=b=luminance;
				}
				else
				{
					var t2:Number = (luminance<=0.5)? luminance*(1+saturation):luminance+saturation-(luminance*saturation);
					var t1:Number = 2*luminance-t2;
					var t3:Vector.<Number> = new Vector.<Number>();
					t3.push(hue+1/3);
					t3.push(hue);
					t3.push(hue-1/3);
					var clr:Vector.<Number> = new Vector.<Number>();
					clr.push(0);
					clr.push(0);
					clr.push(0);
					for(var i:int=0;i<3;i++)
					{
						if(t3[i]<0)
							t3[i]+=1;
						if(t3[i]>1)
							t3[i]-=1;
 
						if(6*t3[i] < 1)
							clr[i]=t1+(t2-t1)*t3[i]*6;
						else if(2*t3[i]<1)
							clr[i]=t2;
						else if(3*t3[i]<2)
							clr[i]=(t1+(t2-t1)*((2/3)-t3[i])*6);
						else
							clr[i]=t1;
					}
					r=clr[0];
					g=clr[1];
					b=clr[2];
				}
			}
			return (int(r*255)<<16)+(int(g*255)<<8)+int(b*255);
		}
	}
 
}