﻿package foras.tools {
	import foras.tools.core.*;
	import flash.display.Sprite;
	/**
	 * ...
	 * @author Marcin Piekaj
	 */
	public class ForFireworksGenerator extends Sprite {
		
		
		private var vFire:Vector.<ForFirework>;
		private var vRocket:Vector.<ForFireworkRocket>;
		private var vBlur:Vector.<ForFireworkBlur>;
		private var forSetup:ForFireworkSettings;
		
		private var nStartX:Number;
		private var nLengthX:Number;
		private var nStartY:Number;
		private var nTime:Number;
		private var nTimeout:Number;
		
		public function ForFireworksGenerator(settings:ForFireworkSettings) {
			forSetup = settings;
			vFire = new Vector.<ForFirework>();
			vRocket = new Vector.<ForFireworkRocket>();
			vBlur = new Vector.<ForFireworkBlur>();
		}
		public function init(minX:Number = 150, maxX:Number = 600, startY:Number = 700, timeout:int = 15):void {
			nTimeout = timeout;
			nTime = 1;
			nStartX = minX;
			nLengthX = maxX - minX;
			nStartY = startY;
			close();
			fireRocket();
		}
		public function process():void {
			graphics.clear();
			
			if(nTime == nTimeout) {
				fireRocket();
				nTime = 0;
			} else nTime++;
			
			var blur:ForFireworkBlur;
			var i:int;
			for(i = 0; i < vBlur.length; i++) {
				blur = vBlur[i];
				if(blur.process(forSetup.rocketBlurStep)) {
					graphics.lineStyle(blur.nSize, blur.nColor, blur.nAlpha);
					graphics.moveTo(blur.nStartX, blur.nStartY);
					graphics.lineTo(blur.nEndX, blur.nEndY);
				} else {
					vBlur.splice(i, 1);
				}
			}
			
			var nx:Number;
			var ny:Number;
			for each(var rocket:ForFireworkRocket in vRocket) {
				nx = rocket.x;
				ny = rocket.y;
				graphics.lineStyle(forSetup.rocketSize, rocket.getColor());
				graphics.moveTo(nx, ny);
				rocket.process(forSetup.rocketGravity);
				graphics.lineTo(rocket.x, rocket.y);
				vBlur.push( new ForFireworkBlur(nx, ny, rocket.x, rocket.y, forSetup.rocketSize, rocket.getColor(), forSetup.rocketBlurAlpha) );
			}
			for each(var fire:ForFirework in vFire) {
				fire.process();
			}
		}
		public function close():void {
			while(vFire.length > 0) removeChild( vFire.shift() );
			vRocket = new Vector.<ForFireworkRocket>();
			vBlur = new Vector.<ForFireworkBlur>();
		}
		
		private function fireRocket():void {
			var rocket:ForFireworkRocket = new ForFireworkRocket(forSetup);
			rocket.x = nStartX + (Math.random() * nLengthX);
			rocket.y = nStartY;
			rocket.init(onRocket);
			vRocket.push(rocket);
		}
		
		//
		// events
		private function onRocket():void {
			var rocket:ForFireworkRocket = vRocket.shift();			
			var fire:ForFirework = new ForFirework(forSetup);
			fire.x = rocket.x;
			fire.y = rocket.y;
			fire.init(onFire);
			vFire.push(fire);
			addChild(fire);
		}
		private function onFire():void {
			removeChild(vFire.shift());
			if(nTime > nTimeout) fireRocket(); //this will only happen when nTimeout was set to 0 on init()
		}
		
	}

}