﻿package foras.tools.core {
	import foras.display.ForHSLColor;
	import flash.display.GradientType;
	import flash.display.SpreadMethod;
	import flash.display.Sprite;
	import flash.geom.Matrix;
	public class ForFirework extends Sprite {
		private static const DRAWING_STEP_MIN:Number = 0.25; // Too low will make some slow moving sparks flicker (best left as it is now)
		
		private var fCall:Function;
		private var vSpark:Vector.<ForFireworkSpark>;
		private var vBlur:Vector.<ForFireworkBlur>;
		private var forSetup:ForFireworkSettings;
		private var hslColor:ForHSLColor;
		
		private var nLife:int;
		private var nStep:int;
		private var nSpeed:Number;
		private var nSize:Number;
		private var nFade:Number;
		private var nColor:Number;
		private var nColorDir:Number;
		private var nGrow:Number;
		
		private var nRotation:Number;
		private var nBlur:Number;
		private var nSplit:int;
		private var nSplitRot:Number;
		private var nGrav:Number;
		
		private var nGradSize:Number;
		private var aGradColor:Array;
		private var aGradAlpha:Array;
		private var aGradRatio:Array;
		private var mGradMatrix:Matrix;
		public function ForFirework(settings:ForFireworkSettings) {
			forSetup = settings;
			nStep = 0;
			nGradSize = 0;
			nLife = settings.burstLife;
			nSpeed = settings.burstSpeed;
			nFade = 1;
			nSize = 0.1 + settings.sparkSize;
			nGrow = settings.sparkGrow;
			
			hslColor = settings.colorHSL.clone();
			nColor = settings.color;
			nColorDir = Math.random() < 0.5 ? -1 : 1;
			
			aGradColor = [nColor, nColor];
			aGradAlpha = [settings.burstFlareAlpha, 0];
			aGradRatio = [0x00, 0xff];
			mGradMatrix = new Matrix();
			
			vSpark = new Vector.<ForFireworkSpark>();
			vBlur = new Vector.<ForFireworkBlur>();
			
			nRotation = settings.burstRotation / nLife;
			nBlur = settings.burstBlur;
			nSplit = int(Math.random()*nLife);
			nSplitRot = settings.burstSplit;
			nGrav = settings.burstGravity;
		}
		public function init(onEnd:Function):void {
			fCall = onEnd;
			var spark:ForFireworkSpark;
			var count:int = forSetup.sparkCount;
			for(var i:int = 0; i < count; i++) {
				spark = new ForFireworkSpark(Math.random() * Math.PI * 2,forSetup.sparkSpread);
				vSpark.push(spark);
			}
		}
		public function process():void {
			nStep ++;
			if(nStep >= nLife) {
				fCall();
				return;
			}
			
			// Calculating current speed curve (creates a nice slowdown)
			var vec:Number = 1 - Math.sin( (nStep / nLife) * Math.PI / 2 );
			if(vec < 0.1 && nFade > 0) {
				nFade = vec*10;
				if(nFade < 0) nFade = 0;
			}
			
			// Clearing all graphics
			graphics.clear();
			
			// Mixing the color
			if(hslColor.Hue + (forSetup.burstColorStep * nColorDir) >= 1) hslColor.Hue = 0;
			else if(hslColor.Hue + (forSetup.burstColorStep * nColorDir) <= 0) hslColor.Hue = 1;
			else hslColor.Hue = hslColor.Hue + (forSetup.burstColorStep * nColorDir);
			nColor = hslColor.getRGB();
			aGradColor = [nColor, nColor];
			
			// background gradient
			nGradSize += vec * nSpeed;
			var totalSize:Number = nGradSize * 2;
			mGradMatrix.createGradientBox(totalSize, totalSize, 0, -nGradSize, -nGradSize);
			
			var alphas:Array = [aGradAlpha[0] * nFade, 0];
			graphics.beginGradientFill(GradientType.RADIAL, aGradColor, alphas, aGradRatio, mGradMatrix, SpreadMethod.PAD);        
			graphics.drawRect( -nGradSize, -nGradSize, totalSize, totalSize);
			
			// Blur
			var blur:ForFireworkBlur;
			var i:int;
			for(i = 0; i < vBlur.length; i++) {
				blur = vBlur[i];
				if(blur.process(forSetup.burstBlurStep)) {
					graphics.lineStyle(blur.nSize, blur.nColor, blur.nAlpha);
					graphics.moveTo(blur.nStartX, blur.nStartY);
					graphics.lineTo(blur.nEndX, blur.nEndY);
				} else {
					vBlur.splice(i, 1);
				}
			}
			
			// Main fireworks
			var spark:ForFireworkSpark;
			var split:ForFireworkSpark;
			var nx:Number = 0;
			var ny:Number = 0;
			
			nSize += nGrow;
			graphics.lineStyle(nSize, nColor, nFade);
			nSplit --;
			if(nSplit == 0) {
				var splitSparks:Vector.<ForFireworkSpark> = new Vector.<ForFireworkSpark>();
				var amount:int = int(vSpark.length * Math.random());
				for(i = 0; i < amount; i++) {
					spark = vSpark[i];
					split = spark.clone();
					split.addRotation( -Math.PI / 4);
					splitSparks.push(split);
					split = spark.clone();
					split.addRotation( Math.PI / 4);
					splitSparks.push(split);
				}
				for(i = amount; i < vSpark.length; i++) {
					splitSparks.push(vSpark[i]);
				}
				vSpark = splitSparks;
			}
			
			for each(spark in vSpark) {
				nx = spark.x;
				ny = spark.y;
				graphics.moveTo(nx, ny);
				spark.addRotation(nRotation);
				spark.addGravity(nGrav);
				spark.process(vec * nSpeed);
				vBlur.push( new ForFireworkBlur(nx, ny, spark.x, spark.y, nSize, nColor, nBlur) );
				graphics.lineTo(
						Math.abs(spark.x - nx) < DRAWING_STEP_MIN ? nx + DRAWING_STEP_MIN : spark.x,
						Math.abs(spark.y - ny) < DRAWING_STEP_MIN ? ny + DRAWING_STEP_MIN : spark.y);
				
			}
			
		}
	}
}