﻿package foras.tools.core {
	import foras.display.ForHSLColor;
	public class ForFireworkSettings {
		
		private static const COLOR_START:ForHSLColor = new ForHSLColor(0, 1, 0.6);	// Hue, Saturation, Luminance (Brightness). From 0 to 1.
		private static const COLOR_END:ForHSLColor = new ForHSLColor(1, 0.8, 0.4);
		
		private static const ROCKET_GRAVITY:Number = 0.15;
		private static const ROCKET_SIZE:Number = 4;				//line width
		private static const ROCKET_SPEED_START:Number = 15;
		private static const ROCKET_SPEED_NOISE:Number = 2;
		private static const ROCKET_LIFE_START:Number = 30;
		private static const ROCKET_LIFE_NOISE:Number = 8;
		private static const ROCKET_BLUR_ALPHA:Number = 0.5;
		private static const ROCKET_BLUR_STEP:Number = 0.05;		//How fast the blur will fade out (lower values = longer fading time)
		private static const ROCKET_SPEED_STOP:Number = 0.1;
		
		private static const BURST_LIFE_START:int = 40;	//in fps
		private static const BURST_LIFE_NOISE:int = 10;
		private static const BURST_SPEED_START:Number = 10;
		private static const BURST_SPEED_NOISE:Number = 2;
		private static const BURST_BLUR_START:Number = 0.5;		//both of those value will determine how visible the blur will be.
		private static const BURST_BLUR_NOISE:Number = 0.0;
		private static const BURST_FLARE_ALPHA:Number = 0.25;		// from 0 to 1
		private static const BURST_ROTATION_START:Number = 0;
		private static const BURST_ROTATION_NOISE:Number = Math.PI/4;
		private static const BURST_ROTATION_SPLIT_START:Number = Math.PI/8;
		private static const BURST_ROTATION_SPLIT_NOISE:Number = Math.PI / 8;
		private static const BURST_GRAVITY_START:Number = 0;
		private static const BURST_GRAVITY_NOISE:Number = 0;
		private static const BURST_COLOR_STEP:Number = 0.02;
		private static const BURST_BLUR_STEP:Number = 0.1;		//How fast the blur will fade out (lower values = longer fading time)
		
		private static const SPARK_SIZE_START:Number = 2;	//Line width
		private static const SPARK_SIZE_NOISE:Number = 1;
		private static const SPARK_SIZE_GROW_START:Number = -0.05;	//Can also be negative to make the line shrink during animation
		private static const SPARK_SIZE_GROW_NOISE:Number = 0.1;
		private static const SPARK_COUNT_MIN:int = 50;
		private static const SPARK_COUNT_NOISE:int = 20;
		private static const SPARK_SPREAD_MIN:Number = 0.25;
		private static const SPARK_SPREAD_NOISE:Number = 0.75;
		
		private var hslColor:ForHSLColor;
		private var nColor:Number;
		
		private var nRocketSize:Number;
		private var nRocketGravity:Number;
		private var nRocketSpeedMin:Number;
		private var nRocketSpeedNoise:Number;
		private var nRocketSpeedStop:Number;
		private var nRocketLifeMin:Number;
		private var nRocketLifeNoise:Number;
		private var nRocketBlurStep:Number;
		private var nRocketBlurAlpha:Number;
		
		private var nBurstColorStep:Number;
		private var nBurstLifeMin:int;
		private var nBurstLifeNoise:int;
		private var nBurstSpeedMin:Number;
		private var nBurstSpeedNoise:Number;
		private var nBurstBlurMin:Number;
		private var nBurstBlurNoise:Number;
		private var nBurstBlurStep:Number;
		private var nBurstFlare:Number;
		private var nBurstRotationMin:Number;
		private var nBurstRotationNoise:Number;
		private var nBurstSplitMin:Number;
		private var nBurstSplitNoise:Number;
		private var nBurstGravityMin:Number;
		private var nBurstGravityNoise:Number;
		
		private var nSparkSizeMin:Number;
		private var nSparkSizeNoise:Number;
		private var nSparkGrowMin:Number;
		private var nSparkGrowNoise:Number;
		private var nSparkCountMin:Number;
		private var nSparkCountNoise:Number;
		private var nSparkSpreadMin:Number;
		private var nSparkSpreadNoise:Number;
		
		public function ForFireworkSettings() {
			nRocketGravity = ROCKET_GRAVITY;
			nRocketSize = ROCKET_SIZE;
			nRocketBlurAlpha = ROCKET_BLUR_ALPHA;
			nRocketBlurStep = ROCKET_BLUR_STEP;
			nRocketSpeedMin = ROCKET_SPEED_START;
			nRocketSpeedNoise = ROCKET_SPEED_NOISE;
			nRocketLifeMin = ROCKET_LIFE_START;
			nRocketLifeNoise = ROCKET_LIFE_NOISE;
			
			nBurstColorStep = BURST_COLOR_STEP;
			nBurstLifeMin = BURST_LIFE_START;
			nBurstLifeNoise = BURST_LIFE_NOISE;
			nBurstSpeedMin = BURST_SPEED_START;
			nBurstSpeedNoise = BURST_SPEED_NOISE;
			nBurstBlurMin = BURST_BLUR_START;
			nBurstBlurNoise = BURST_BLUR_NOISE;
			nBurstBlurStep = BURST_BLUR_STEP;
			nBurstFlare = BURST_FLARE_ALPHA;
			nBurstRotationMin = BURST_ROTATION_START;
			nBurstRotationNoise = BURST_ROTATION_NOISE;
			nBurstSplitMin = BURST_ROTATION_SPLIT_START;
			nBurstSplitNoise = BURST_ROTATION_SPLIT_NOISE;
			nBurstGravityMin = BURST_GRAVITY_START;
			nBurstGravityNoise = BURST_GRAVITY_NOISE;
		
			nSparkSizeMin = SPARK_SIZE_START;
			nSparkSizeNoise = SPARK_SIZE_NOISE;
			nSparkGrowMin = SPARK_SIZE_GROW_START;
			nSparkGrowNoise = SPARK_SIZE_GROW_NOISE;
			nSparkCountMin = SPARK_COUNT_MIN;
			nSparkCountNoise = SPARK_COUNT_NOISE;
			nSparkSpreadMin = SPARK_SPREAD_MIN;
			nSparkSpreadNoise = SPARK_SPREAD_NOISE;
			
			createNewColor(COLOR_START,COLOR_END);
		}
		
		public function createNewColor(colorStart:ForHSLColor,colorEnd:ForHSLColor):void {
			var h:Number = colorStart.Hue + ((colorEnd.Hue-colorStart.Hue) * Math.random());
			var s:Number = colorStart.Saturation + ((colorEnd.Saturation-colorStart.Saturation) * Math.random());
			var l:Number = colorStart.Luminance + ((colorEnd.Luminance-colorStart.Luminance) * Math.random());
			hslColor = new ForHSLColor(h, s, l);
			nColor = hslColor.getRGB();
		}
		public function get color():Number { return nColor; }
		public function set color(n:Number):void { nColor = n; }
		public function get colorHSL():ForHSLColor { return hslColor; }
		public function set colorHSL(hsl:ForHSLColor):void { hslColor = hsl; }
		
		public function get rocketBlurAlpha():Number { return nRocketBlurAlpha; }
		public function set rocketBlurAlpha(n:Number):void { nRocketBlurAlpha = n; }
		
		public function get rocketBlurStep():Number { return nRocketBlurStep; }
		public function set rocketBlurStep(n:Number):void { nRocketBlurStep = n; }
		
		public function get rocketSize():Number { return nRocketSize; }
		public function set rocketSize(n:Number):void { nRocketSize = n; }
		
		public function get rocketGravity():Number { return nRocketGravity; }
		public function set rocketGravity(n:Number):void { nRocketGravity = n; }
		
		public function get rocketSpeed():Number { return nRocketSpeedMin + (Math.random() * nRocketSpeedNoise); }
		public function set rocketSpeedMin(n:Number):void { nRocketSpeedMin = n; }
		public function set rocketSpeedNoise(n:Number):void { nRocketLifeNoise = n; }
		
		public function get rocketSpeedStop():Number { return nRocketSpeedStop; }
		public function set rocketSpeedStop(n:Number):void { nRocketSpeedStop = n; }
		
		public function get rocketLife():Number { return nRocketLifeMin + (Math.random() * nRocketLifeNoise); }
		public function set rocketLifeMin(n:Number):void { nRocketLifeMin = n; }
		public function set rocketLifeNoise(n:Number):void { nRocketLifeNoise = n; }
		
		public function get burstColorStep():Number { return nBurstColorStep; }
		public function set burstColorStep(n:Number):void { nBurstColorStep = n; }
		
		public function get burstLife():Number { return nBurstLifeMin + (Math.random() * nBurstLifeNoise); }
		public function set burstLifeMin(n:Number):void { nBurstLifeMin = n; }
		public function set burstLifeNoise(n:Number):void { nBurstLifeNoise = n; }
		
		public function get burstSpeed():Number { return nBurstSpeedMin + (Math.random() * nBurstSpeedNoise); }
		public function set burstSpeedMin(n:Number):void { nBurstSpeedMin = n; }
		public function set burstSpeedNoise(n:Number):void { nBurstSpeedNoise = n; }
		
		public function get burstBlur():Number { return nBurstBlurMin + (Math.random() * nBurstBlurNoise); }
		public function set burstBlurMin(n:Number):void { nBurstBlurMin = n; }
		public function set burstBlurNoise(n:Number):void { nBurstBlurNoise = n; }
		
		public function get burstBlurStep():Number { return nBurstBlurStep; }
		public function set burstBlurStep(n:Number):void { nBurstBlurStep = n; }
		
		public function get burstFlareAlpha():Number { return nBurstFlare; }
		public function set burstFlareAlpha(n:Number):void { nBurstFlare = n; }
			
		public function get burstRotation():Number { return nBurstRotationMin + (Math.random() * nBurstRotationNoise); }
		public function set burstRotationMin(n:Number):void { nBurstRotationMin = n; }
		public function set burstRotationNoise(n:Number):void { nBurstRotationNoise = n; }
		
		public function get burstSplit():Number { return nBurstSplitMin + (Math.random() * nBurstSplitNoise); }
		public function set burstSplitMin(n:Number):void { nBurstSplitMin = n; }
		public function set burstSplitNoise(n:Number):void { nBurstSplitNoise = n; }
		
		public function get burstGravity():Number { return nBurstGravityMin + (Math.random() * nBurstGravityNoise); }
		public function set burstGravityMin(n:Number):void { nBurstGravityMin = n; }
		public function set burstGravityNoise(n:Number):void { nBurstGravityNoise = n; }
		
		public function get sparkSize():Number { return nSparkSizeMin + (Math.random() * nSparkSizeNoise); }
		public function set sparkSizeMin(n:Number):void { nSparkSizeMin = n; }
		public function set sparkSizeNoise(n:Number):void { nSparkSizeNoise = n; }
		
		public function get sparkGrow():Number { return nSparkGrowMin + (Math.random() * nSparkGrowNoise); }
		public function set sparkGrowMin(n:Number):void { nSparkGrowMin = n; }
		public function set sparkGrowNoise(n:Number):void { nSparkGrowNoise = n; }
		
		public function get sparkCount():Number { return nSparkCountMin + (Math.random() * nSparkCountNoise); }
		public function set sparkCountMin(n:Number):void { nSparkCountMin = n; }
		public function set sparkCountNoise(n:Number):void { nSparkCountNoise = n; }
		
		public function get sparkSpread():Number { return nSparkSpreadMin + (Math.random() * nSparkSpreadNoise); }
		public function set sparkSpreadMin(n:Number):void { nSparkSpreadMin = n; }
		public function set sparkSpreadNoise(n:Number):void { nSparkSpreadNoise = n; }
		
	}
}