﻿package foras.utils {
	
	import foras.utils.core.ForListEntry;
	
	public class ForLinkedList {

		public var head:ForListEntry;
		public var tail:ForListEntry;
		public var length:int;

		public function ForLinkedList(list:ForListEntry = null) {
			length = 0;
			recreate(list);
		}
		public function recreate(list:ForListEntry):void {
			if(list == null) return;
			head = list;
			tail = list;
			length = 1;
			
			var entry:ForListEntry = list;
			while(entry.next!=null) {
				entry = entry.next;
				tail = entry;
				length ++;
			}
		}
		
		public function push(elem:Object):void {
			if(tail==null) {
				head = new ForListEntry(elem);
				tail = head;
				length = 1;
			} else {
				tail.next = new ForListEntry(elem,tail);
				tail = tail.next;
				length ++;
			}
		}
		public function pop():Object {
			if(tail==null) return null;
			var o:Object = tail.item;
			tail = tail.prev;
			tail.next = null;
			length --;
			return o;
		}
		
		public function shift(elem:Object):void {
			if(head==null) {
				head = new ForListEntry(elem);
				tail = head;
				length = 1;
			} else {
				head.prev = new ForListEntry(elem,null,head);
				head = head.prev;
				length ++;
			}
		}
		public function unshift():Object {
			if(head==null) return null;
			var o:Object = head.item;
			head = head.next;
			head.prev = null;
			length --;
			return o;
		}
		
		public function removeByIndex(index:int):void {
			if (tail == null || index>=length) return;
			var i_head:int = 0;
			var itr_head:ForListEntry = head;
			var i_tail:int = 0;
			var itr_tail:ForListEntry = tail;
			while (i_head < index || i_tail > index) {
				// from head
				i_head ++;
				itr_head = itr_head.next;
				// from tail
				i_tail --;
				itr_tail = itr_tail.prev;
			}
			
			if (i_head == index) remove(itr_head);
			else if (i_tail == index) remove(itr_tail);
		}
		public function removeByValue(item:*):void {
			if (tail == null) return;
			var itr_head:ForListEntry = head;
			var itr_tail:ForListEntry = tail;
			while (itr_head != null && itr_tail != null) {
				// from head
				if (item == itr_head.item) {
					remove(itr_head);
					return;
				}
				itr_head = itr_head.next;
				
				//from tail
				if (item == itr_tail.item) {
					remove(itr_tail);
					return;
				}
				itr_tail = itr_tail.prev;
			}
		}
		private function remove(entry:ForListEntry):void {
			if (entry == tail) {
				if (tail.prev != null) {
					tail = tail.prev;
				} else {
					tail = head = null;
				}
			} else if (entry == head) {
				if (head.next != null) {
					head = head.next;
				} else {
					tail = head = null;
				}
			} 
			release(entry);
			length --;
		}
		
		public function get(index:int):* {
			if (tail == null || index>=length) return;
			var i_head:int = 0;
			var itr_head:ForListEntry = head;
			var i_tail:int = length;
			var itr_tail:ForListEntry = tail;
			while (i_head < index && i_tail > index) {
				// from head
				i_head ++;
				itr_head = itr_head.next;
				// from tail
				i_tail --;
				itr_tail = itr_tail.prev;
			}
			
			if (i_head == index) {
				return itr_head.item;
			} else if (i_tail == index) {
				return itr_tail.item
			}
		}
		
		public function sort(compare:Function):void {
			recreate( ForListEntry.sort(head,compare) );
		}
		
		public function toString():String {
			if(head == null) return "[]";
			var s:String = "["+head;
			
			var itr:ForListEntry = head.next;
			while(itr!=null) {
				s+=","+itr;
				itr = itr.next;
			}
			s += "]";
			return s;
			
		}
		
		
		
		public static function insertAfter(target:ForListEntry,obj:Object):void {
			var elem:ForListEntry = new ForListEntry(obj);
			elem.next = target.next;
			elem.prev = target;
			target.next = elem;
			if(elem.next!=null) elem.next.prev = elem;
		}
		public static function release(target:ForListEntry):void {
			if(target.next!=null) target.next.prev = target.prev;
			if(target.prev!=null) target.prev.next = target.next;
			target.next = null;
			target.prev = null;
		}
		public static function replace(target:ForListEntry,elem:ForListEntry):void {
			elem.next = target.next;
			elem.prev = target.prev;
			if(target.next!=null) target.next.prev = elem;
			if(target.prev!=null) target.prev.next = elem;
			target.next = null;
			target.prev = null;
		}
		
		public static function createFrom(array:Array):ForLinkedList {
			var list:ForLinkedList = new ForLinkedList();
			for( var i:int = 0; i < array.length; i++ ) {
				list.push(array[i]);
			}
			return list;
		}

	}
	
}
